#ifndef DIRECTSOUND_H
#define DIRECTSOUND_H

#include <dsound.h>
#include <windows.h>
#include <assert.h>


struct DX_SOUND
{
	lpdSoundBuff;
	LPDIRECTSOUND lpdSound;
	DSBUFFERDESC dsBuffDesc;
};


bool DX_SoundParseWav(void *file, WAVEFORMATEX **WaveHeader, BYTE **WaveData,DWORD &WaveLength);
bool InitDXSound(DX_SOUND&, HWND);
bool LoadDXSound(DX_SOUND&, char *file_name);
void FreeDXSound(DX_SOUND&);



bool DX_SoundParseWav(void *file, WAVEFORMATEX **wavHeader, BYTE **wavData, DWORD &wavLength)
{
	LPDWORD		pFile;
	LPDWORD		pFileEnd;
	DWORD		length;
	DWORD		type;

	pFile = (DWORD*)file;

	if((DWORD)*pFile++ != mmioFOURCC('R', 'I', 'F', 'F'))
		return false;	
	
	length = *pFile++;
	type = *pFile++;

	if(type != mmioFOURCC('W', 'A', 'V', 'E'))
		return false;

	pFileEnd = (DWORD*)((BYTE*)pFile + length - 4);

	while(pFile < pFileEnd)
	{
		type = *pFile++;
		length = *pFile++;

		switch(type)
		{
			case mmioFOURCC('f', 'm', 't', ' '):
				if(wavHeader && !*wavHeader)
				{
					*wavHeader = (WAVEFORMATEX*)pFile;
					if(wavLength != 0)
						return true;
				}
				
				break;

			case mmioFOURCC('d', 'a', 't', 'a'):
			
				if((wavData && !*wavData))
				{
					*wavData = (LPBYTE)pFile;
					wavLength = length;
					if(*wavHeader) 
						return true;
				}
				
				break;
		
		} // end of switch(type)

		pFile = (DWORD*)((BYTE*)pFile + ((length + 1) & ~1));

	} // end of while(pFile < pFileEnd)

	return false; 

} // bool DX_SoundParseWav(void *file, WAVEFORMATEX **wavHeader, BYTE **wavData, 
  //					   DWORD &wavLength)
                          
bool InitDXSound(DX_SOUND &dx_sound, HWND win_hwnd)
{
	DWORD result;

	result = DirectSoundCreate(NULL, &(dx_sound.lpdSound), NULL);
		
	if(result != DS_OK){return false;}
	
	result = dx_sound.lpdSound->SetCooperativeLevel(win_hwnd,DSSCL_PRIORITY);
		
	if(result != DS_OK){return false;}

	return true; // We got loaded :)
}


bool LoadDXSound(DX_SOUND &dx_sound, char *file_name)
{
	HANDLE				file_handle = NULL;
	HANDLE				file_mapping = NULL;
	PUCHAR				file_pointer = NULL;

	file_handle = CreateFile(file_name, GENERIC_READ, FILE_SHARE_READ, NULL,
							 OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE | 
							 FILE_FLAG_SEQUENTIAL_SCAN,	(HANDLE)NULL);
			
	if(file_handle == INVALID_HANDLE_VALUE){return false;}

	file_mapping = CreateFileMapping(file_handle, NULL,	PAGE_READONLY, 0, 0, NULL);
		
		// Error Check
		if(file_mapping == NULL)
		{
			CloseHandle(file_handle);
				return false;
		}

	file_pointer = (PUCHAR)MapViewOfFile(file_mapping, FILE_MAP_READ, 0, 0, 0);
		
		// Error Check
		if(file_pointer == NULL)
		{
			CloseHandle(file_handle);
			CloseHandle(file_mapping);
				return false;
		}


	WAVEFORMATEX		*wavHeader = NULL;
	DWORD				wavLength = 0;
	
	BYTE *wavData = NULL;

	if(DX_SoundParseWav(file_pointer, &wavHeader, &wavData, wavLength) == false)
	{
	
		CloseHandle(file_handle);
		CloseHandle(file_mapping);
		UnmapViewOfFile(file_pointer);
			return false;
	}

	if(dx_sound.dsBuffDesc.dwSize != 0)
		memset(&(dx_sound.dsBuffDesc),0,sizeof(DSBUFFERDESC));

	// Fill the dsBuffDesc
	dx_sound.dsBuffDesc.dwSize = sizeof(DSBUFFERDESC);
	dx_sound.dsBuffDesc.dwFlags = DSBCAPS_LOCSOFTWARE;
	dx_sound.dsBuffDesc.dwBufferBytes = wavLength;
	dx_sound.dsBuffDesc.lpwfxFormat = wavHeader;

	DWORD result;

	if(dx_sound.lpdSoundBuff)
	{
		dx_sound.lpdSoundBuff->Release();
		dx_sound.lpdSoundBuff = NULL;
	}

	result = dx_sound.lpdSound->CreateSoundBuffer(&(dx_sound.dsBuffDesc),
												  &(dx_sound.lpdSoundBuff),
												  NULL);
		// Error Check
		if(result != DS_OK)
		{
		
			CloseHandle(file_handle);
			CloseHandle(file_mapping);
			UnmapViewOfFile(file_pointer);
				return false;
		}

	BYTE *soundPointer = NULL;
	
	DWORD spLength = 0;

	result = dx_sound.lpdSoundBuff->Lock(0,dx_sound.dsBuffDesc.dwBufferBytes,
										 (LPVOID*)&soundPointer,&spLength,
									     NULL,NULL,DSBLOCK_FROMWRITECURSOR);
	
		// Error Check
		if(result != DS_OK)
		{
			
			CloseHandle(file_handle);
			CloseHandle(file_mapping);
			UnmapViewOfFile(file_pointer);
				return false;
		}

	memcpy(soundPointer,wavData,spLength);

	dx_sound.lpdSoundBuff->Unlock((LPVOID*)soundPointer,spLength,NULL,NULL);

	
	CloseHandle(file_handle);
	CloseHandle(file_mapping);
	UnmapViewOfFile(file_pointer);

	return true; // We got loaded :)

} // end of LoadSound()

void FreeDXSound(DX_SOUND &dx_sound)
{

	if(dx_sound.lpdSoundBuff){dx_sound.lpdSoundBuff->Release();}
	if(dx_sound.lpdSound){dx_sound.lpdSound->Release();}
	memset(&dx_sound,0,sizeof(DX_SOUND));

}

#endif